@@ -31,6 +31,7 @@ extern Tk_EventProc *xctk_dragfilebox(ClientData, XEvent *);
 extern int XcTagCallback(Tcl_Interp *, int, Tcl_Obj *CONST objv[]);
 extern Tcl_Obj *evaluate_raw(oparamptr, int *);
 extern char *TCL_to_PS(char *);
+extern XCWindowData *GUI_init(int, Tcl_Obj *CONST objv[]);
 #endif
 
 extern char *evaluate_expr(oparamptr, objinstptr);
@@ -143,7 +144,8 @@ extern void nextarccycle(arcptr, short);
 extern void buttonhandler(xcWidget, caddr_t, XButtonEvent *);
 extern void keyhandler(xcWidget, caddr_t, XKeyEvent *);
 extern Boolean compatible_function(int);
-extern void eventdispatch(int, int, int);
+extern int eventdispatch(int, int, int);
+extern int functiondispatch(int, short, int, int);
 extern void releasehandler(xcWidget, caddr_t, XKeyEvent *);
 extern void setsnap(short);
 extern void snapelement();
@@ -403,27 +405,26 @@ extern void printhelp(xcWidget);
 
 /* from keybindings.c */
 
-extern int firstbinding(int);
-extern Boolean ismacro(int);
-extern Boolean isbound(int, int);
-extern Boolean isnbound(int, int, short *);
-extern int boundfunction(int, int);
+extern int firstbinding(xcWidget, int);
+extern Boolean ismacro(xcWidget, int);
+extern int boundfunction(xcWidget, int, short *);
 extern int string_to_func(const char *, short *);
 extern int string_to_key(const char *);
-extern char *binding_to_string(int, int);
+extern char *function_binding_to_string(xcWidget, int);
+extern char *key_binding_to_string(xcWidget, int);
 extern char *func_to_string(int);
 extern char *key_to_string(int);
 extern void printeditbindings();
-extern int add_vbinding(int, int, short);
-extern int add_binding(int, int);
-extern int add_keybinding(const char *, const char *);
+extern int add_vbinding(xcWidget, int, int, short);
+extern int add_binding(xcWidget, int, int);
+extern int add_keybinding(xcWidget, const char *, const char *);
 extern void default_keybindings();
-extern int remove_binding(int, int);
-extern void remove_keybinding(const char *, const char *);
+extern int remove_binding(xcWidget, int, int);
+extern void remove_keybinding(xcWidget, const char *, const char *);
 
 #ifndef TCL_WRAPPER
-extern void mode_rebinding(int, int);
-extern void mode_tempbinding(int, int);
+extern void mode_rebinding(xcWidget, int, int);
+extern void mode_tempbinding(xcWidget, int, int);
 #endif
 
 /* from libraries.c: */
@@ -478,7 +479,7 @@ extern void setkern(xcWidget, stringpart *);
 extern void getdscale(xcWidget, caddr_t, caddr_t);
 extern void setdscale(xcWidget, XPoint *);
 extern labelptr gettextsize(float **);
-extern void gettsize(xcWidget, caddr_t, caddr_t);
+/* Xt */ extern void gettsize(xcWidget, caddr_t, caddr_t);
 extern void setosize(xcWidget, objinstptr);
 extern void getosize(xcWidget, caddr_t, caddr_t);
 extern void getwirewidth(xcWidget, caddr_t, caddr_t);
@@ -497,10 +498,10 @@ extern void toolcursor(int);
 extern void promptparam(xcWidget, caddr_t, caddr_t);
 #endif
 extern void stringparam(xcWidget, caddr_t, caddr_t);
-extern void toggle(xcWidget, Boolean *, caddr_t);
+extern void toggle(xcWidget, pointertype, Boolean *);
 extern int setelementstyle(xcWidget, u_short, u_short);
 extern void setcolorscheme(Boolean);
-extern void inversecolor(xcWidget, Boolean *, caddr_t);
+extern void inversecolor(xcWidget, pointertype, caddr_t);
 extern void togglegrid(u_short);
 extern float parseunits(char *);
 extern void setgridtype(char *);
