@@ -37,13 +37,11 @@
 /* Exported Variable definitions					*/
 /*----------------------------------------------------------------------*/
 
-extern short eventmode;	/* keep track of the mode the screen is in */
 extern Display	*dpy;
 extern int *appcolors;
 extern Cursor	appcursors[NUM_CURSORS];
-extern Clientdata areastruct;
+extern XCWindowData *areawin;
 extern char _STR[150];
-extern short attachto;
 extern short textpos, textend;
 
 /*----------------------------------------------------------------------*/
@@ -53,12 +51,12 @@ extern short textpos, textend;
 void selectfilter(xcWidget w, pointertype value, caddr_t calldata)
 {
    short bitwise = (short)value;
-   Boolean bval = (areastruct.filter & bitwise) ? True : False;
+   Boolean bval = (areawin->filter & bitwise) ? True : False;
 
    if (bval)
-      areastruct.filter &= ~bitwise;
+      areawin->filter &= ~bitwise;
    else
-      areastruct.filter |= bitwise;
+      areawin->filter |= bitwise;
 
 #ifndef TCL_WRAPPER
    toggle(w, &bval, calldata);
@@ -73,14 +71,14 @@ Boolean checkselect(short value)
 {
    short *check;
  
-   value &= areastruct.filter;	/* apply the selection filter */
+   value &= areawin->filter;	/* apply the selection filter */
 
-   if (areastruct.selects == 0) select_element(value);
-   if (areastruct.selects == 0) return False;
-   for (check = areastruct.selectlist; check < areastruct.selectlist +
-	areastruct.selects; check++)
+   if (areawin->selects == 0) select_element(value);
+   if (areawin->selects == 0) return False;
+   for (check = areawin->selectlist; check < areawin->selectlist +
+	areawin->selects; check++)
       if (SELECTTYPE(check) & value) break;
-   if (check == areastruct.selectlist + areastruct.selects) return False;
+   if (check == areawin->selectlist + areawin->selects) return False;
    else return True;
 }
 
@@ -136,16 +134,16 @@ void geneasydraw(short instance, int mode, objectptr curobj, objinstptr curinst)
 
 void gendrawselected(short *newselect, objectptr curobj, objinstptr curinst)
 {
-   SetFunction(dpy, areastruct.gc, GXcopy);
-   SetForeground(dpy, areastruct.gc, BACKGROUND);
+   SetFunction(dpy, areawin->gc, GXcopy);
+   SetForeground(dpy, areawin->gc, BACKGROUND);
    geneasydraw(*newselect, DOFORALL, curobj, curinst);
    indicateparams(*(curobj->plist + *newselect));
 
-   SetFunction(dpy, areastruct.gc, GXxor);
-   SetForeground(dpy, areastruct.gc, SELECTCOLOR ^ BACKGROUND);
+   SetFunction(dpy, areawin->gc, GXxor);
+   SetForeground(dpy, areawin->gc, SELECTCOLOR ^ BACKGROUND);
    geneasydraw(*newselect, DOFORALL, curobj, curinst);
 
-   SetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
+   SetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
    indicateparams(*(curobj->plist + *newselect));
 }
 
@@ -157,14 +155,14 @@ short *allocselect()
 {
    short *newselect;
 
-   if (areastruct.selects == 0)
-      areastruct.selectlist = (short *) malloc(sizeof(short));
+   if (areawin->selects == 0)
+      areawin->selectlist = (short *) malloc(sizeof(short));
    else
-      areastruct.selectlist = (short *) realloc(areastruct.selectlist,
-	 (areastruct.selects + 1) * sizeof(short));
+      areawin->selectlist = (short *) realloc(areawin->selectlist,
+	 (areawin->selects + 1) * sizeof(short));
 
-   newselect = areastruct.selectlist + areastruct.selects;
-   areastruct.selects++;
+   newselect = areawin->selectlist + areawin->selects;
+   areawin->selects++;
 
    return newselect;
 }
@@ -178,16 +176,16 @@ void setoptionmenu()
    short      *mselect;
    labelptr   mlabel;
 
-   if (areastruct.selects == 0) {
-      setallstylemarks(areastruct.style);
-      setcolormark(areastruct.color);
+   if (areawin->selects == 0) {
+      setallstylemarks(areawin->style);
+      setcolormark(areawin->color);
       setdefaultfontmarks();
       setparammarks(NULL);
       return;
    }
 
-   for (mselect = areastruct.selectlist; mselect < areastruct.selectlist +
-	areastruct.selects; mselect++) {
+   for (mselect = areawin->selectlist; mselect < areawin->selectlist +
+	areawin->selects; mselect++) {
       setcolormark(SELTOCOLOR(mselect));
       setparammarks(SELTOGENERIC(mselect));
       switch(SELECTTYPE(mselect)) {
@@ -252,13 +250,13 @@ Boolean pathselect(genericptr *curgen, short class, float range)
    /*		is an empirical result.					   */
    /*----------------------------------------------------------------------*/
 
-   float	wirelim = 2 + range / (*areastruct.vscale + 0.05);
+   float	wirelim = 2 + range / (areawin->vscale + 0.05);
    long		sqrwirelim = (int)(wirelim * wirelim);
 
    long		newdist;
    Boolean	selected = False;
 
-   class &= areastruct.filter;	/* apply the selection filter */
+   class &= areawin->filter;	/* apply the selection filter */
 
    if ((*curgen)->type == (class & ARC)) {
 
@@ -275,11 +273,11 @@ Boolean pathselect(genericptr *curgen, short class, float range)
 	 nearpt[1].y = nearpt[0].y;
 	 nearpt[0].x = (short)(currentpt->x);
 	 nearpt[0].y = (short)(currentpt->y);
-	 newdist = finddist(&nearpt[0], &nearpt[1], &areastruct.save);
+	 newdist = finddist(&nearpt[0], &nearpt[1], &areawin->save);
          if (newdist <= sqrwirelim) break;
       }
       if ((!(TOARC(curgen)->style & UNCLOSED)) && (newdist > sqrwirelim))
-	 newdist = finddist(&nearpt[0], &nearpt[2], &areastruct.save);
+	 newdist = finddist(&nearpt[0], &nearpt[2], &areawin->save);
 
       if (newdist <= sqrwirelim) selected = True;
    }
@@ -294,7 +292,7 @@ Boolean pathselect(genericptr *curgen, short class, float range)
       nearpt[0].x = (short)(TOSPLINE(curgen)->points[0].x);
       nearpt[0].y = (short)(TOSPLINE(curgen)->points[0].y);
       newdist = finddist(&(TOSPLINE(curgen)->ctrl[0]), &(nearpt[0]),
-		   &areastruct.save);
+		   &areawin->save);
       if (newdist > sqrwirelim) {
          for (currentpt = TOSPLINE(curgen)->points; currentpt <
 		  TOSPLINE(curgen)->points + INTSEGS; currentpt++) {
@@ -302,15 +300,15 @@ Boolean pathselect(genericptr *curgen, short class, float range)
 	    nearpt[1].y = nearpt[0].y;
 	    nearpt[0].x = (short)(currentpt->x);
 	    nearpt[0].y = (short)(currentpt->y);
-	    newdist = finddist(&nearpt[0], &nearpt[1], &areastruct.save);
+	    newdist = finddist(&nearpt[0], &nearpt[1], &areawin->save);
             if (newdist <= sqrwirelim) break;
 	 }
 	 if (newdist > sqrwirelim) {
 	    newdist = finddist(&nearpt[0], &(TOSPLINE(curgen)->ctrl[3]),
-			&areastruct.save);
+			&areawin->save);
             if ((!(TOSPLINE(curgen)->style & UNCLOSED)) && (newdist > sqrwirelim))
 	       newdist = finddist(&(TOSPLINE(curgen)->ctrl[0]),
-		     &(TOSPLINE(curgen)->ctrl[3]), &areastruct.save);
+		     &(TOSPLINE(curgen)->ctrl[3]), &areawin->save);
 	 }
       }
 
@@ -325,12 +323,12 @@ Boolean pathselect(genericptr *curgen, short class, float range)
 
       for (currentpt = TOPOLY(curgen)->points; currentpt < TOPOLY(curgen)
 		->points + TOPOLY(curgen)->number - 1; currentpt++) {
-	 newdist = finddist(currentpt, currentpt + 1, &areastruct.save);
+	 newdist = finddist(currentpt, currentpt + 1, &areawin->save);
 	 if (newdist <= sqrwirelim) break;
       }
       if ((!(TOPOLY(curgen)->style & UNCLOSED)) && (newdist > sqrwirelim))
 	 newdist = finddist(currentpt, TOPOLY(curgen)->points,
-		&areastruct.save);
+		&areawin->save);
 
       if (newdist <= sqrwirelim) selected = True;
    }
@@ -361,7 +359,7 @@ selection *genselectelement(short class, u_char mode, objectptr selobj,
       /* when selection element to attach to, don't select the element */
       /* to be attached. . .					       */
 
-      if (attachto == 1 && eventmode != COPY_MODE && eventmode !=
+      if (areawin->attachto == TRUE && eventmode != COPY_MODE && eventmode !=
 	    MOVE_MODE && curgen == EDITPART) continue;
 
       selected = False;
@@ -394,7 +392,7 @@ selection *genselectelement(short class, u_char mode, objectptr selobj,
 	
             /* check for point inside bounding box, as for objects */
 
-	    selected = test_insideness(areastruct.save.x, areastruct.save.y,
+	    selected = test_insideness(areawin->save.x, areawin->save.y,
 		newboxpts);
 	    if (selected) textpos = textend = 0;
 	 }
@@ -408,7 +406,7 @@ selection *genselectelement(short class, u_char mode, objectptr selobj,
 	 graphicbbox(curg, newboxpts);
 
          /* check for point inside bounding box, as for objects */
-	 selected = test_insideness(areastruct.save.x, areastruct.save.y,
+	 selected = test_insideness(areawin->save.x, areawin->save.y,
 		newboxpts);
       }
 
@@ -425,7 +423,7 @@ selection *genselectelement(short class, u_char mode, objectptr selobj,
 		TOPATH(curgen)->parts; pathp++)
 	    if (pathselect(pathp, SPLINE|ARC|POLYGON, range)) {
 	       selected = True;
-	       areastruct.editsubpart = (short)(pathp - TOPATH(curgen)->plist);
+	       areawin->editsubpart = (short)(pathp - TOPATH(curgen)->plist);
 	       break;
 	    }
       }
@@ -441,7 +439,7 @@ selection *genselectelement(short class, u_char mode, objectptr selobj,
          /*  point's y-position is the same for all bbox segments if the   */
          /*  pointer position is inside the bounding box.		   */
 
-	 selected = test_insideness(areastruct.save.x, areastruct.save.y,
+	 selected = test_insideness(areawin->save.x, areawin->save.y,
 		newboxpts);
       }
 
@@ -481,10 +479,10 @@ Boolean areaelement(genericptr *curgen)
       case(ARC):
 	   /* check center of arcs */
 
-      	   selected = (TOARC(curgen)->position.x < areastruct.save.x) &&
-	  	(TOARC(curgen)->position.x > areastruct.origin.x) &&
-		(TOARC(curgen)->position.y < areastruct.save.y) &&
-		(TOARC(curgen)->position.y > areastruct.origin.y);
+      	   selected = (TOARC(curgen)->position.x < areawin->save.x) &&
+	  	(TOARC(curgen)->position.x > areawin->origin.x) &&
+		(TOARC(curgen)->position.y < areawin->save.y) &&
+		(TOARC(curgen)->position.y > areawin->origin.y);
 	   break;
 
       case(POLYGON):
@@ -493,9 +491,9 @@ Boolean areaelement(genericptr *curgen)
 	   selected = False;
            for (currentpt = TOPOLY(curgen)->points; currentpt <
 		TOPOLY(curgen)->points + TOPOLY(curgen)->number; currentpt++) {
-              if ((currentpt->x < areastruct.save.x) && (currentpt->x >
-	           areastruct.origin.x) && (currentpt->y < areastruct.save.y) &&
-	           (currentpt->y > areastruct.origin.y)) {
+              if ((currentpt->x < areawin->save.x) && (currentpt->x >
+	           areawin->origin.x) && (currentpt->y < areawin->save.y) &&
+	           (currentpt->y > areawin->origin.y)) {
 	         selected = True;
 		 break;
 	      }
@@ -505,14 +503,14 @@ Boolean areaelement(genericptr *curgen)
       case(SPLINE):
 	   /* check each control point of the spline */
 	
-           selected = ((TOSPLINE(curgen)->ctrl[0].x < areastruct.save.x) && 
-	   	(TOSPLINE(curgen)->ctrl[0].x > areastruct.origin.x) &&
-		(TOSPLINE(curgen)->ctrl[0].y < areastruct.save.y) &&
-		(TOSPLINE(curgen)->ctrl[0].y > areastruct.origin.y))
-		|| ((TOSPLINE(curgen)->ctrl[3].x < areastruct.save.x) &&
-		(TOSPLINE(curgen)->ctrl[3].x > areastruct.origin.x) &&
-		(TOSPLINE(curgen)->ctrl[3].y < areastruct.save.y) &&
-		(TOSPLINE(curgen)->ctrl[3].y > areastruct.origin.y));
+           selected = ((TOSPLINE(curgen)->ctrl[0].x < areawin->save.x) && 
+	   	(TOSPLINE(curgen)->ctrl[0].x > areawin->origin.x) &&
+		(TOSPLINE(curgen)->ctrl[0].y < areawin->save.y) &&
+		(TOSPLINE(curgen)->ctrl[0].y > areawin->origin.y))
+		|| ((TOSPLINE(curgen)->ctrl[3].x < areawin->save.x) &&
+		(TOSPLINE(curgen)->ctrl[3].x > areawin->origin.x) &&
+		(TOSPLINE(curgen)->ctrl[3].y < areawin->save.y) &&
+		(TOSPLINE(curgen)->ctrl[3].y > areawin->origin.y));
 	   break;
    }
    return selected;
@@ -532,17 +530,17 @@ void selectarea()
 
    /* put points of area bounding box into proper order */
 
-   if (areastruct.origin.y > areastruct.save.y) {
+   if (areawin->origin.y > areawin->save.y) {
       short tmp;
-      tmp = areastruct.origin.y;
-      areastruct.origin.y = areastruct.save.y;
-      areastruct.save.y = tmp;
+      tmp = areawin->origin.y;
+      areawin->origin.y = areawin->save.y;
+      areawin->save.y = tmp;
    }
-   if (areastruct.origin.x > areastruct.save.x) {
+   if (areawin->origin.x > areawin->save.x) {
       short tmp;
-      tmp = areastruct.origin.x;
-      areastruct.origin.x = areastruct.save.x;
-      areastruct.save.x = tmp;
+      tmp = areawin->origin.x;
+      areawin->origin.x = areawin->save.x;
+      areawin->save.x = tmp;
    }
 
    textpos = textend = 0;
@@ -550,23 +548,23 @@ void selectarea()
 	+ topobject->parts; curgen++) {
 
       /* apply the selection filter */
-      if (!((*curgen)->type & areastruct.filter)) continue;
+      if (!((*curgen)->type & areawin->filter)) continue;
 
       switch(ELEMENTTYPE(*curgen)) {
 	case(OBJINST):
            /* check for instance of object center point inside area box */
-           selected = (TOOBJINST(curgen)->position.x < areastruct.save.x) &&
-		(TOOBJINST(curgen)->position.x > areastruct.origin.x) &&
-		(TOOBJINST(curgen)->position.y < areastruct.save.y) &&
-		(TOOBJINST(curgen)->position.y > areastruct.origin.y);
+           selected = (TOOBJINST(curgen)->position.x < areawin->save.x) &&
+		(TOOBJINST(curgen)->position.x > areawin->origin.x) &&
+		(TOOBJINST(curgen)->position.y < areawin->save.y) &&
+		(TOOBJINST(curgen)->position.y > areawin->origin.y);
 	   break;
 
 	case(GRAPHIC):
            /* check for graphic image center point inside area box */
-           selected = (TOGRAPHIC(curgen)->position.x < areastruct.save.x) &&
-		(TOGRAPHIC(curgen)->position.x > areastruct.origin.x) &&
-		(TOGRAPHIC(curgen)->position.y < areastruct.save.y) &&
-		(TOGRAPHIC(curgen)->position.y > areastruct.origin.y);
+           selected = (TOGRAPHIC(curgen)->position.x < areawin->save.x) &&
+		(TOGRAPHIC(curgen)->position.x > areawin->origin.x) &&
+		(TOGRAPHIC(curgen)->position.y < areawin->save.y) &&
+		(TOGRAPHIC(curgen)->position.y > areawin->origin.y);
 	   break;
 
         case(LABEL): {
@@ -581,23 +579,22 @@ void selectarea()
 	   if (slab->string->type != FONT_NAME) break;
 
 	   /* create a 4-point polygon out of the select box information */
-	   bboxpts[0].x = bboxpts[3].x = areastruct.origin.x;
-	   bboxpts[0].y = bboxpts[1].y = areastruct.origin.y;
-	   bboxpts[1].x = bboxpts[2].x = areastruct.save.x;
-	   bboxpts[2].y = bboxpts[3].y = areastruct.save.y;
+	   bboxpts[0].x = bboxpts[3].x = areawin->origin.x;
+	   bboxpts[0].y = bboxpts[1].y = areawin->origin.y;
+	   bboxpts[1].x = bboxpts[2].x = areawin->save.x;
+	   bboxpts[2].y = bboxpts[3].y = areawin->save.y;
 
    	   /* translate select box into the coordinate system of the label */
 	   InvTransformPoints(bboxpts, newboxpts, 4, slab->position,
 		  slab->scale, slab->rotation);
 
 	   if (slab->pin) {
-	      if (!areastruct.schemon) continue;
 	      for (j = 0; j < 4; j++) 
 		 pinadjust(slab->justify, &(newboxpts[j].x),
 			&(newboxpts[j].y), -1);
 	   }
 
-	   tmpext = ULength(slab->string, areastruct.topinstance, 0.0, 0, NULL);
+	   tmpext = ULength(slab->string, areawin->topinstance, 0.0, 0, NULL);
 	   adj.x = (slab->justify & NOTLEFT ? (slab->justify & RIGHT ? 
 			tmpext.width : tmpext.width >> 1) : 0);
 	   adj.y = (slab->justify & NOTBOTTOM ? (slab->justify & TOP ? 
@@ -608,12 +605,12 @@ void selectarea()
 	   /* do an insideness test with the select box.		*/
 
 	   state = tmpl2 = 0;
-	   for (j = 0; j < stringlength(slab->string, True, areastruct.topinstance); j++) {
-	      strptr = findstringpart(j, &locpos, slab->string, areastruct.topinstance);
+	   for (j = 0; j < stringlength(slab->string, True, areawin->topinstance); j++) {
+	      strptr = findstringpart(j, &locpos, slab->string, areawin->topinstance);
 	      if (locpos < 0) continue;	  /* only look at printable characters */
 	      if (strptr->type == RETURN) tmpl2 = 0;
 	      tmpl1 = tmpl2;
-	      tmpext = ULength(slab->string, areastruct.topinstance, 0.0, j + 1, NULL);
+	      tmpext = ULength(slab->string, areawin->topinstance, 0.0, j + 1, NULL);
 	      tmpl2 = tmpext.width;
 	      isect = test_insideness(((tmpl1 + tmpl2) >> 1) - adj.x,
 			(tmpext.base + (BASELINE >> 1)) - adj.y, newboxpts);
@@ -657,8 +654,8 @@ void selectarea()
       /* check if this part has already been selected */
 
       if (selected)
-         for (newselect = areastruct.selectlist; newselect <
-              areastruct.selectlist + areastruct.selects; newselect++)
+         for (newselect = areawin->selectlist; newselect <
+              areawin->selectlist + areawin->selects; newselect++)
             if (*newselect == (short)(curgen - topobject->plist))
                   selected = False;
 
@@ -673,14 +670,14 @@ void selectarea()
 
    /* if none or > 1 label has been selected, cancel any textpos placement */
 
-   if (!checkselect(LABEL) || areastruct.selects != 1 ||
-	(areastruct.selects == 1 && SELECTTYPE(areastruct.selectlist) != LABEL)) {
+   if (!checkselect(LABEL) || areawin->selects != 1 ||
+	(areawin->selects == 1 && SELECTTYPE(areawin->selectlist) != LABEL)) {
       textpos = textend = 0;
    }
 
    /* Register the selection as an undo event */
-   register_for_undo(XCF_Select, UNDO_DONE, areastruct.topinstance,
-		areastruct.selectlist, areastruct.selects);
+   register_for_undo(XCF_Select, UNDO_DONE, areawin->topinstance,
+		areawin->selectlist, areawin->selects);
 
    /* Drawing of selected objects will take place when drawarea() is */
    /* executed after the button release.			     */
@@ -693,9 +690,9 @@ void selectarea()
 void startdesel(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
    if (eventmode == NORMAL_MODE) {
-      if (areastruct.selects == 0)
+      if (areawin->selects == 0)
 	 Wprintf("Nothing to deselect!");
-      else if (areastruct.selects == 1)
+      else if (areawin->selects == 1)
 	 unselect_all();
    }
 }
@@ -708,8 +705,9 @@ void draw_all_selected()
 {
    int j;
 
-   for (j = 0; j < areastruct.selects; j++)
-      gendrawselected(areastruct.selectlist + j, topobject, areastruct.topinstance);
+   if (areawin->hierstack != NULL) return;
+   for (j = 0; j < areawin->selects; j++)
+      gendrawselected(areawin->selectlist + j, topobject, areawin->topinstance);
 }
 
 /*---------------------------------------------------------*/
@@ -720,14 +718,15 @@ void draw_normal_selected(objectptr thisobj, objinstptr thisinst)
 {
    short *lastselect;
 
-   if (areastruct.selects == 0) return;
+   if (areawin->selects == 0) return;
+   else if (areawin->hierstack != NULL) return;
 
    /* reset the graphics state */
 
-   SetFunction(dpy, areastruct.gc, GXcopy);
+   SetFunction(dpy, areawin->gc, GXcopy);
 
-   for (lastselect = areastruct.selectlist; lastselect < areastruct.selectlist 
-	+ areastruct.selects; lastselect++) {
+   for (lastselect = areawin->selectlist; lastselect < areawin->selectlist 
+	+ areawin->selects; lastselect++) {
       XTopSetForeground(SELTOCOLOR(lastselect)); 
       geneasydraw(*lastselect, SELTOCOLOR(lastselect), thisobj, thisinst);
    }
@@ -761,9 +760,10 @@ static void freeselection(selection *rselect)
 
 void freeselects()
 {
-   if (areastruct.selects > 0)
-      free(areastruct.selectlist);
-   areastruct.selects = 0;
+   if (areawin->selects > 0)
+      free(areawin->selectlist);
+   areawin->selects = 0;
+   free_stack(&areawin->hierstack);
 }
 
 /*--------------------------------------------------------------*/
@@ -773,10 +773,10 @@ void freeselects()
 
 void clearselects_noundo()
 {
-   if (areastruct.selects > 0) {
+   if (areawin->selects > 0) {
       freeselects();
-      setallstylemarks(areastruct.style);
-      setcolormark(areastruct.color);
+      setallstylemarks(areawin->style);
+      setcolormark(areawin->color);
       setdefaultfontmarks();
    }
 }
@@ -787,8 +787,8 @@ void clearselects_noundo()
 
 void clearselects()
 {
-   if (areastruct.selects > 0) {
-      register_for_undo(XCF_Select, UNDO_DONE, areastruct.topinstance,
+   if (areawin->selects > 0) {
+      register_for_undo(XCF_Select, UNDO_DONE, areawin->topinstance,
 		NULL, 0);
       clearselects_noundo();
    }
@@ -801,7 +801,7 @@ void clearselects()
 
 void unselect_all()
 {
-   draw_normal_selected(topobject, areastruct.topinstance);
+   draw_normal_selected(topobject, areawin->topinstance);
    clearselects();
 }
 
@@ -836,7 +836,7 @@ selection *recurselect(short class, u_char mode, pushlistptr *seltop)
    selinst = (*seltop)->thisinst;
    selobj = selinst->thisobject;
 
-   class &= areastruct.filter;		/* apply the selection filter */
+   class &= areawin->filter;		/* apply the selection filter */
 
    unselects = 0;
    rselect = genselectelement(class, locmode, selobj, selinst);
@@ -853,27 +853,27 @@ selection *recurselect(short class, u_char mode, pushlistptr *seltop)
          selnew->next = NULL;
          (*seltop)->next = selnew;
 	 
-         /* Translate areastruct.save into object's own coordinate system */
-         savesave.x = areastruct.save.x;
-         savesave.y = areastruct.save.y;
-         InvTransformPoints(&areastruct.save, &tmppt, 1, selinst->position,
+         /* Translate areawin->save into object's own coordinate system */
+         savesave.x = areawin->save.x;
+         savesave.y = areawin->save.y;
+         InvTransformPoints(&areawin->save, &tmppt, 1, selinst->position,
 			selinst->scale, selinst->rotation);
-         areastruct.save.x = tmppt.x;
-         areastruct.save.y = tmppt.y;
+         areawin->save.x = tmppt.x;
+         areawin->save.y = tmppt.y;
          /* Fprintf(stdout, "objinst %s found in object %s; searching recursively\n",
 			selinst->thisobject->name, selobj->name); */
          /* Fprintf(stdout, "cursor position originally (%d, %d); "
 			"in new object is (%d, %d)\n",
 			savesave.x, savesave.y,
-			areastruct.save.x, areastruct.save.y); */
+			areawin->save.x, areawin->save.y); */
          UPushCTM();
          UPreMultCTM(DCTM, selinst->position, selinst->scale, selinst->rotation);
 	 rclass = class;
 	 if ((class & (~OBJINST)) == 0) class = ALL_TYPES;
          rcheck = recurselect(class, recmode, &selnew);
          UPopCTM();
-         areastruct.save.x = savesave.x;
-         areastruct.save.y = savesave.y;
+         areawin->save.x = savesave.x;
+         areawin->save.y = savesave.y;
 
 	 /* If rgen is NULL, remove selected object from the list, and	*/
 	 /* remove the last entry from the pushlist stack.		*/
@@ -918,15 +918,15 @@ selection *recurselect(short class, u_char mode, pushlistptr *seltop)
 void startselect()
 {
    eventmode = SELAREA_MODE;
-   areastruct.origin.x = areastruct.save.x;
-   areastruct.origin.y = areastruct.save.y;
-   UDrawBox(areastruct.origin, areastruct.save);
+   areawin->origin.x = areawin->save.x;
+   areawin->origin.y = areawin->save.y;
+   UDrawBox(areawin->origin, areawin->save);
 
 #ifdef TCL_WRAPPER
-   Tk_CreateEventHandler(areastruct.area, ButtonMotionMask,
+   Tk_CreateEventHandler(areawin->area, ButtonMotionMask,
 		(Tk_EventProc *)xctk_drag, NULL);
 #else
-   xcAddEventHandler(areastruct.area, ButtonMotionMask, False,
+   xcAddEventHandler(areawin->area, ButtonMotionMask, False,
 		(xcEventHandler)xlib_drag, NULL);
 #endif
 
@@ -942,13 +942,13 @@ void trackselarea()
    u_int  nullui;
 
    newpos = UGetCursorPos();
-   if (newpos.x == areastruct.save.x && newpos.y == areastruct.save.y) return;
+   if (newpos.x == areawin->save.x && newpos.y == areawin->save.y) return;
 
-   UDrawBox(areastruct.origin, areastruct.save);
-   UDrawBox(areastruct.origin, newpos);
+   UDrawBox(areawin->origin, areawin->save);
+   UDrawBox(areawin->origin, newpos);
 
-   areastruct.save.x = newpos.x;
-   areastruct.save.y = newpos.y;
+   areawin->save.x = newpos.x;
+   areawin->save.y = newpos.y;
 }
 
 /*----------------------*/
@@ -961,13 +961,13 @@ void trackrescale()
    u_int  nullui;
 
    newpos = UGetCursorPos();
-   if (newpos.x == areastruct.save.x && newpos.y == areastruct.save.y) return;
+   if (newpos.x == areawin->save.x && newpos.y == areawin->save.y) return;
 
-   UDrawRescaleBox(&areastruct.save);
+   UDrawRescaleBox(&areawin->save);
    UDrawRescaleBox(&newpos);
 
-   areastruct.save.x = newpos.x;
-   areastruct.save.y = newpos.y;
+   areawin->save.x = newpos.x;
+   areawin->save.y = newpos.y;
 }
 
 /*----------------------------------------------------------------------*/
@@ -980,8 +980,8 @@ int dcompare(const void *a, const void *b)
    genericptr agen, bgen;
    short j, k, adist, bdist;
 
-   cpt.x = areastruct.save.x;
-   cpt.y = areastruct.save.y;
+   cpt.x = areawin->save.x;
+   cpt.y = areawin->save.y;
 
    j = *((short *)a);
    k = *((short *)b);
@@ -1037,7 +1037,7 @@ short *recurse_select_element(short class, u_char mode) {
    Boolean unselect = False;
 
    seltop = (pushlistptr)malloc(sizeof(pushlist));
-   seltop->thisinst = areastruct.topinstance;
+   seltop->thisinst = areawin->topinstance;
    seltop->next = NULL;
 
    /* Definition for unselecting an element */
@@ -1069,30 +1069,30 @@ short *recurse_select_element(short class, u_char mode) {
       ilast = -1;
       k = 0;
       for (i = 0; i < rselect->selects; i++) {
-	 for (j = 0; j < areastruct.selects; j++) {
-	    if (*(areastruct.selectlist + j) == *(rselect->selectlist + i)) {
+	 for (j = 0; j < areawin->selects; j++) {
+	    if (*(areawin->selectlist + j) == *(rselect->selectlist + i)) {
 	       jlast = j;
 	       ilast = i;
 	       if (++k == localpick)
 	          break;
 	    }
 	 }
-	 if (j < areastruct.selects) break;
+	 if (j < areawin->selects) break;
       }
       if (ilast >= 0) {
 	 newselect = rselect->selectlist + ilast;
-         SetFunction(dpy, areastruct.gc, GXcopy);
-         XTopSetForeground(GSELTOCOLOR(topobject, newselect));
-         geneasydraw(*newselect, DEFAULTCOLOR, topobject, areastruct.topinstance);
-         areastruct.selects--;
-         for (k = jlast; k < areastruct.selects; k++)
-	    *(areastruct.selectlist + k) = *(areastruct.selectlist + k + 1);
+         SetFunction(dpy, areawin->gc, GXcopy);
+         XTopSetForeground(SELTOCOLOR(newselect));
+         geneasydraw(*newselect, DEFAULTCOLOR, topobject, areawin->topinstance);
+         areawin->selects--;
+         for (k = jlast; k < areawin->selects; k++)
+	    *(areawin->selectlist + k) = *(areawin->selectlist + k + 1);
 
-         if (areastruct.selects == 0) freeselects();
+         if (areawin->selects == 0) freeselects();
 
          /* Register the selection as an undo event */
-         register_for_undo(XCF_Select, mode, areastruct.topinstance,
-		areastruct.selectlist, areastruct.selects);
+         register_for_undo(XCF_Select, mode, areawin->topinstance,
+		areawin->selectlist, areawin->selects);
       }
    }
 
@@ -1106,11 +1106,11 @@ short *recurse_select_element(short class, u_char mode) {
       ilast = -1;
       k = 0;
       for (i = 0; i < rselect->selects; i++) {
-	 for (j = 0; j < areastruct.selects; j++) {
-	    if (*(areastruct.selectlist + j) == *(rselect->selectlist + i))
+	 for (j = 0; j < areawin->selects; j++) {
+	    if (*(areawin->selectlist + j) == *(rselect->selectlist + i))
 	       break;
 	 }
-	 if (j == areastruct.selects) {
+	 if (j == areawin->selects) {
 	    ilast = i;
 	    if (++k == localpick)
 	       break;
@@ -1120,15 +1120,15 @@ short *recurse_select_element(short class, u_char mode) {
       if (ilast >= 0) {
          newselect = allocselect();
          *newselect = *(rselect->selectlist + ilast);
-         gendrawselected(newselect, topobject, areastruct.topinstance);
+         gendrawselected(newselect, topobject, areawin->topinstance);
          setoptionmenu();
-         u2u_snap(&areastruct.save);
+         u2u_snap(&areawin->save);
 
          /* Register the selection as an undo event	*/
 	 /* (only if selection changed)			*/
 
-         register_for_undo(XCF_Select, mode, areastruct.topinstance,
-		areastruct.selectlist, areastruct.selects);
+         register_for_undo(XCF_Select, mode, areawin->topinstance,
+		areawin->selectlist, areawin->selects);
       }
    }
 
@@ -1143,5 +1143,5 @@ short *recurse_select_element(short class, u_char mode) {
    freeselection(saveselect);
    saveselect = rselect;
 
-   return areastruct.selectlist;
+   return areawin->selectlist;
 }
